"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var function_1 = require("fp-ts/lib/function");
var Option_1 = require("fp-ts/lib/Option");
var Record_1 = require("fp-ts/lib/Record");
var pipeable_1 = require("fp-ts/lib/pipeable");
var io_ts_1 = require("io-ts");
var querystring_1 = require("querystring");
var url_1 = require("url");
var Either_1 = require("fp-ts/lib/Either");
/**
 * @since 0.4.0
 */
var Route = /** @class */ (function () {
    function Route(parts, query) {
        this.parts = parts;
        this.query = query;
    }
    Route.prototype.toString = function (encode) {
        if (encode === void 0) { encode = true; }
        var nonUndefinedQuery = pipeable_1.pipe(this.query, Record_1.filter(function (value) { return value !== undefined; }));
        var qs = querystring_1.stringify(nonUndefinedQuery);
        var parts = encode ? this.parts.map(encodeURIComponent) : this.parts;
        return '/' + parts.join('/') + (qs ? '?' + qs : '');
    };
    Route.empty = new Route([], {});
    Route.isEmpty = function (r) { return r.parts.length === 0 && Record_1.isEmpty(r.query); };
    Route.parse = function (s, decode) {
        if (decode === void 0) { decode = true; }
        var route = url_1.parse(s, true);
        var oparts = Option_1.option.map(Option_1.fromNullable(route.pathname), function (s) {
            var r = s.split('/').filter(Boolean);
            return decode ? r.map(decodeURIComponent) : r;
        });
        var parts = Option_1.isNone(oparts) ? [] : oparts.value;
        return new Route(parts, route.query);
    };
    return Route;
}());
exports.Route = Route;
var assign = function (a) { return function (b) { return Object.assign({}, a, b); }; };
/**
 * @since 0.4.0
 */
var Parser = /** @class */ (function () {
    function Parser(run) {
        this.run = run;
    }
    Parser.prototype.map = function (f) {
        return this.chain(function (a) { return Parser.of(f(a)); }); // <= derived
    };
    Parser.prototype.ap = function (fab) {
        var _this = this;
        return fab.chain(function (f) { return _this.map(f); }); // <= derived
    };
    Parser.prototype.chain = function (f) {
        var _this = this;
        return new Parser(function (r) { return Option_1.option.chain(_this.run(r), function (_a) {
            var a = _a[0], r2 = _a[1];
            return f(a).run(r2);
        }); });
    };
    Parser.prototype.alt = function (that) {
        var _this = this;
        return new Parser(function (r) {
            var oar = _this.run(r);
            return Option_1.isNone(oar) ? that.run(r) : oar;
        });
    };
    /** A mapped Monoidal.mult */
    Parser.prototype.then = function (that) {
        return that.ap(this.map(assign));
    };
    Parser.of = function (a) { return new Parser(function (s) { return Option_1.some(function_1.tuple(a, s)); }); };
    return Parser;
}());
exports.Parser = Parser;
/**
 * @since 0.4.0
 */
function zero() {
    return new Parser(function () { return Option_1.none; });
}
exports.zero = zero;
/**
 * @since 0.4.0
 */
function parse(parser, r, a) {
    var oa = Option_1.option.map(parser.run(r), function (_a) {
        var a = _a[0];
        return a;
    });
    return Option_1.isNone(oa) ? a : oa.value;
}
exports.parse = parse;
/**
 * @since 0.4.0
 */
function format(formatter, a, encode) {
    if (encode === void 0) { encode = true; }
    return formatter.run(Route.empty, a).toString(encode);
}
exports.format = format;
/**
 * @since 0.4.0
 */
var Formatter = /** @class */ (function () {
    function Formatter(run) {
        this.run = run;
    }
    Formatter.prototype.contramap = function (f) {
        var _this = this;
        return new Formatter(function (r, b) { return _this.run(r, f(b)); });
    };
    Formatter.prototype.then = function (that) {
        var _this = this;
        return new Formatter(function (r, ab) { return that.run(_this.run(r, ab), ab); });
    };
    return Formatter;
}());
exports.Formatter = Formatter;
/**
 * @since 0.4.0
 */
var Match = /** @class */ (function () {
    function Match(parser, formatter) {
        this.parser = parser;
        this.formatter = formatter;
    }
    Match.prototype.imap = function (f, g) {
        return new Match(this.parser.map(f), this.formatter.contramap(g));
    };
    Match.prototype.then = function (that) {
        var p = this.parser.then(that.parser);
        var f = this.formatter.then(that.formatter);
        return new Match(p, f);
    };
    return Match;
}());
exports.Match = Match;
var singleton = function (k, v) {
    var _a;
    return (_a = {}, _a[k] = v, _a);
};
/**
 * `succeed` matches everything but consumes nothing
 * @since 0.4.0
 */
function succeed(a) {
    return new Match(new Parser(function (r) { return Option_1.some(function_1.tuple(a, r)); }), new Formatter(function_1.identity));
}
exports.succeed = succeed;
/**
 * `end` matches the end of a route
 * @since 0.4.0
 */
exports.end = new Match(new Parser(function (r) { return (Route.isEmpty(r) ? Option_1.some(function_1.tuple({}, r)) : Option_1.none); }), new Formatter(function_1.identity));
/**
 * `type` matches any io-ts type path component
 * @since 0.4.0
 */
function type(k, type) {
    return new Match(new Parser(function (r) {
        if (r.parts.length === 0) {
            return Option_1.none;
        }
        else {
            var head = r.parts[0];
            var tail_1 = r.parts.slice(1);
            return Option_1.option.map(Option_1.fromEither(type.decode(head)), function (a) { return function_1.tuple(singleton(k, a), new Route(tail_1, r.query)); });
        }
    }), new Formatter(function (r, o) { return new Route(r.parts.concat(type.encode(o[k])), r.query); }));
}
exports.type = type;
/**
 * `str` matches any string path component
 * @since 0.4.0
 */
function str(k) {
    return type(k, io_ts_1.string);
}
exports.str = str;
/**
 * @internal
 */
exports.IntegerFromString = new io_ts_1.Type('IntegerFromString', function (u) { return io_ts_1.Int.is(u); }, function (u, c) {
    return Either_1.either.chain(io_ts_1.string.validate(u, c), function (s) {
        var n = +s;
        return isNaN(n) || !Number.isInteger(n) ? io_ts_1.failure(s, c) : io_ts_1.success(n);
    });
}, String);
/**
 * `int` matches any integer path component
 * @since 0.4.0
 */
function int(k) {
    return type(k, exports.IntegerFromString);
}
exports.int = int;
/**
 * `lit(x)` will match exactly the path component `x`
 * For example, `lit('x')` matches `/x`
 * @since 0.4.0
 */
function lit(literal) {
    return new Match(new Parser(function (r) {
        if (r.parts.length === 0) {
            return Option_1.none;
        }
        else {
            var head = r.parts[0];
            var tail = r.parts.slice(1);
            return head === literal ? Option_1.some(function_1.tuple({}, new Route(tail, r.query))) : Option_1.none;
        }
    }), new Formatter(function (r, n) { return new Route(r.parts.concat(literal), r.query); }));
}
exports.lit = lit;
/**
 * @since 0.4.0
 */
function query(type) {
    return new Match(new Parser(function (r) { return Option_1.option.map(Option_1.fromEither(type.decode(r.query)), function (query) { return function_1.tuple(query, new Route(r.parts, {})); }); }), new Formatter(function (r, query) { return new Route(r.parts, type.encode(query)); }));
}
exports.query = query;
//# sourceMappingURL=index.js.map